import gzip
import os
import re
import sys

def replace_uid_in_gzip_file(file_path, old_uid_values, new_uid_values):
    directory, file_name = os.path.split(file_path)
    base_name, ext = os.path.splitext(file_name)
    new_file_path = os.path.join(directory, base_name + '_Synplant2.als')
    
    if os.path.exists(new_file_path):
        return 0
        
    try:
        with gzip.GzipFile(file_path, 'rb') as f:
            original_data = f.read()
    except gzip.BadGzipFile:
        print(f'Error: File {file_path} is not a gzipped file.')
        return 0  # Return 0 replacements made

    # Decompress the content
    decompressed_data = original_data.decode('utf-8', errors='ignore')

    old_uid_pattern = re.compile(r'''
        <Uid> \s*
        <Fields\.0 \s+ Value="{}" \s* /> \s*
        <Fields\.1 \s+ Value="{}" \s* /> \s*
        <Fields\.2 \s+ Value="{}" \s* /> \s*
        <Fields\.3 \s+ Value="{}" \s* /> \s*
        </Uid>
    '''.format(*old_uid_values), re.VERBOSE)

    new_uid = f'''
    <Uid>
        <Fields.0 Value="{new_uid_values[0]}" />
        <Fields.1 Value="{new_uid_values[1]}" />
        <Fields.2 Value="{new_uid_values[2]}" />
        <Fields.3 Value="{new_uid_values[3]}" />
    </Uid>
    '''.strip()

    modified_data, replacements_made = old_uid_pattern.subn(new_uid, decompressed_data)

    if replacements_made > 0:
        with gzip.open(new_file_path, 'wt', encoding='utf-8') as f:
            f.write(modified_data)

    return replacements_made

def find_and_replace_in_directory(directory_path, old_uid_values, new_uid_values):
    report = {}
    for root, dirs, files in os.walk(directory_path):
        for file in files:
            if file.endswith('.als'):
                file_path = os.path.join(root, file)
                replacements_made = replace_uid_in_gzip_file(file_path, old_uid_values, new_uid_values)
                if replacements_made > 0:
                    report[file_path] = replacements_made
    return report

if len(sys.argv) > 1:
    directory_path = sys.argv[1]
    old_uid_values = [1448301646, 1968394358, 1936994911, 1718185061]
    new_uid_values = [1448301646, 1968394355, 2037280876, 1634628608]
    report = find_and_replace_in_directory(directory_path, old_uid_values, new_uid_values)
    if not report:
        print("No new update project files were generated.")
    else:
        print("Generated new project files with Synplant version 1 replaced by version 2, for the following projects:")
        for file_path, replacements_made in report.items():
            replacements_made //= 2
            print(f"{file_path}: {replacements_made} {('replacement', 'replacements')[replacements_made > 1]}")
else:
    print("Usage: python updateToSynplant2.py <directory_path>")
